from random import Random

SEED = "CODETENT"  # Randomizer base seed
MAX_RAND = 1000000
MIN_RAND = 1
BILL_LEN = 6  # Bill code length

my_random = Random()  # not to use shared random module

# Example of a starting state of the database
db = {
    "codes": {
        "ha": 1000,
        "er": 2000,
        "hi": 5000,
        "ta": 10000,
        "qs": 20000
    },
    "cards": {
        "card12345": {
            "pin": "1234",
            "balance": 200000
        },
        "card0001": {
            "pin": "4242",
            "balance": 1000000
        }
    },
    "ha": [
        "ha4851",
        "ha5781",
        "ha0001",
        "ha3921"
    ],
    "er": [
        "er3214",
        "er0002",
        "er0001",
        "er0120"
    ],
    "hi": [
        "hi3123",
        "hi0001",
        "hi3182",
        "hi0010"
    ],
    "ta": [
        "ta0124",
        "ta6001",
        "ta0122",
        "ta0230"
    ],
    "qs": [
        "qs0224",
        "qs6231",
        "qs0222",
        "qs1230"
    ]
}
cards = db["cards"]
codes = db["codes"]


# Set the randomizer to the base seed
def resync():
    my_random.seed(SEED)


# card is a string with format "cardXXXX"
def card_exists(card):
    return card in cards


# In unsafe mode, check with pin. In safe mode, check with otp (refer to docs)
def valid_pin(card, otp, safe=True):
    if not card_exists(card):
        return False

    if not safe:
        return cards[card]["pin"] == otp

    h = hash(cards[card]["pin"])
    r = my_random.randint(MIN_RAND, MAX_RAND)

    return otp == (h + r)


# Change pin; Returns True on successful change
def change_pin(card, otp, new_pin, safe=True):
    if type(new_pin) != str:
        return False
    if len(new_pin) != 4:
        return False
    if not new_pin.isnumeric:
        return False
    if not valid_pin(card, otp, safe):
        return False

    cards[card]["pin"] = new_pin
    return True


# Get balance as int; returns None if unauthorized
def get_balance(card, otp, safe=True):
    if not valid_pin(card, otp, safe):
        return None

    return cards[card]["balance"]


# Check if 'bill' refers to a valid bill code within the bank
def valid_bill(bill):
    if not bill:
        return False
    if type(bill) != str:
        return False
    if len(bill) != 6:
        return False
    if bill[:2] not in codes:
        return False

    return bill in db[bill[:2]]


# Deposit 'amount' money; returns True on success
def deposit(card, otp, amount, safe=True):
    if not valid_pin(card, otp, safe):
        return False
    if not isinstance(amount, int):
        return False
    if amount <= 0:    
        return False

    cards[card]["balance"] += amount
    return True


# Check if account has at least 'amount' money, etc.
def can_withdraw(card, otp, amount, safe=True):
    if not valid_pin(card, otp, safe):
        return False
    if not isinstance(amount, int):
        return False
    if amount <= 0:
        return False
    return cards[card]["balance"] >= amount


# Withdraw 'amount' money; return True on success
def withdraw(card, otp, amount, safe=True):
    if not can_withdraw(card, otp, amount, safe):
        return False

    cards[card]["balance"] -= amount

    assert cards[card]["balance"] >= 0
    return True