import os
import subprocess
import json
from difflib import unified_diff

# Python file name
EXECUTABLE = "python michael.py"

# test files directory
TEST_DIR = "tests"

# print differences between two files
def print_diff(file1_path, file2_path):
    with open(file1_path, 'r') as file1, open(file2_path, 'r') as file2:
        diff = list(unified_diff(file1.readlines(), file2.readlines()))
        print('\n'.join(diff))

def main():
    test_files = sorted(os.listdir(TEST_DIR))
    passed = 0
    
    # .json files are test files
    for test_file in test_files:
        if not test_file.endswith(".json"):
            continue

        # Extract the test name
        test_name = os.path.splitext(test_file)[0]

        # Run test
        with open(os.path.join(TEST_DIR, test_file)) as json_file:

            # Load test json
            cases = json.load(json_file)

            # Initialize appropriate file names for I/O
            input_file = os.path.join(TEST_DIR, f"../{test_name}.in")
            ans_file = os.path.join(TEST_DIR, f"../{test_name}.out")
            got_file = os.path.join(TEST_DIR, f"../{test_name}.got")    


            # Load test cases from json to input and output file
            f_in = open(input_file, "w")
            f_out = open(ans_file, "w")
            for case in cases:
                case_in = case["in"]
                case_out = case["out"]

                # case_in with trailing new line
                if case_in:
                    f_in.write(case_in + "\n")

                # case_out lines, last line doesn't have new line
                for i in range(len(case_out)):
                    f_out.write(case_out[i])
                    if i < len(case_out) - 1:
                        f_out.write("\n")
            # Tell program to stop
            f_in.write("EXIT" + '\n')

            # Close .in .out files
            f_in.close()
            f_out.close()

            # Run the program with input and save the output using subprocess
            with open(input_file, "r") as input_data, open(got_file, "w") as actual_output:
                subprocess.run(EXECUTABLE, stdin=input_data, stdout=actual_output, text=True, shell=True)

            # Compare the actual output with the expected output
            with open(got_file, "r") as actual_output, open(ans_file, "r") as expected_output:
                if actual_output.read() == expected_output.read():
                    passed += 1
                    print(f"{test_name} passed")
                else:
                    print(f"{test_name} failed. ")
                    print_diff(ans_file, got_file)
            
            # Remove generated test files
            os.remove(input_file)
            os.remove(ans_file)
            os.remove(got_file)
    
    print(f"-- score: {passed}/{len(test_files)}")

if __name__ == "__main__":
    main()
